package ictag.gui;

import ictag.domain.Book;
import ictag.util.Resource;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

public class BookTableModel extends AbstractTableModel {
	private	String[]	header = { 
			Resource.getString("book.ictagID"), 
			Resource.getString("book.ISBN"),
			Resource.getString("book.title"),
			Resource.getString("book.author"),
			Resource.getString("book.publisher"),
			Resource.getString("book.shelfID")
			};
	private List<Book>	bookList = new ArrayList<Book>();
	
	/**
	 * コンストラクタ
	 * @param books	本のリスト
	 */
	public BookTableModel(List<Book> books) {
		this.bookList = books;
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return header.length;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return bookList.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	public String getColumnName(int columnIndex) {
		return header[columnIndex];
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return bookList.get(rowIndex).getIcTag();
		case 1:
			return bookList.get(rowIndex).getIsbn();
		case 2:
			return bookList.get(rowIndex).getTitle();
		case 3:
			return bookList.get(rowIndex).getAuthor();
		case 4:
			return bookList.get(rowIndex).getPublisher();
		case 5:
			return bookList.get(rowIndex).getShelfId();
		default:
			return null;
		}
	}

	/**
	 * 本のリストをセットする
	 * @param bookList	本のリスト
	 */
	public void setBookList(List<Book> bookList) {
		this.bookList = bookList;
		fireTableDataChanged();
	}
	
	/**
	 * リストに本を追加する
	 * @param book	本
	 */
	public void addBook(Book book) {
		bookList.add(book);
		fireTableDataChanged();
	}

	/**
	 * 指定されたインデックスの本を返す
	 * @param index	リスト上でのインデックス
	 * @return	本
	 */
	public Book getBook(int index) {
		return (bookList.get(index));
	}
}
