package ictag.gui;

import ictag.HF04SL;
import ictag.domain.Book;
import ictag.util.DataBaseHelper;
import ictag.util.Resource;

import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.MouseInputAdapter;

public class IcTagFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JMenuBar jJMenuBar = null;
	private JMenu jFileMenu = null;
	private JMenuItem jFindItem = null;
	private JMenuItem jResiterItem = null;
	private JMenuItem jDisplayItem = null;
	private JMenuItem jQuitItem = null;
	private JScrollPane jScrollPane = null;
	private JTable jTable = null;
	
	//
	protected	BookTableModel	tableModel = new BookTableModel(new ArrayList<Book>());
	protected	HF04SL			ictag;
	protected	IcTagFrame		icTagFrame;
	
	/**
	 * This is the default constructor
	 */
	public IcTagFrame() {
		this(null);
	}
	
	public IcTagFrame(HF04SL ictag) {
		super();
		icTagFrame = this;
		this.ictag = ictag;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(400, 200);
		this.setJMenuBar(getJJMenuBar());
		this.setContentPane(getJContentPane());
		this.setTitle(Resource.getString("frame.title"));
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getJFileMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes jFileMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getJFileMenu() {
		if (jFileMenu == null) {
			jFileMenu = new JMenu();
			jFileMenu.setText(Resource.getString("menu.file"));
			jFileMenu.add(getJFindItem());
			jFileMenu.add(getJResiterItem());
			jFileMenu.add(getJDisplayItem());
			jFileMenu.add(getJQuitItem());
		}
		return jFileMenu;
	}

	/**
	 * This method initializes jFindItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJFindItem() {
		if (jFindItem == null) {
			jFindItem = new JMenuItem();
			jFindItem.setText(Resource.getString("menu.find"));
			jFindItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					String findString = JOptionPane.showInputDialog(icTagFrame, "Input Serarch String:");
					if (findString != null) {
						try {
							List<Book> bookList =DataBaseHelper.selectBook("%"+findString+"%");
							if (bookList != null)
								tableModel.setBookList(bookList);
						}
						catch (SQLException ex) {
							ex.printStackTrace();
						}
					}
				}
			});
		}
		return jFindItem;
	}

	/**
	 * This method initializes jResiterItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJResiterItem() {
		if (jResiterItem == null) {
			jResiterItem = new JMenuItem();
			jResiterItem.setText(Resource.getString("menu.resister"));
			jResiterItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					BookDialog dialog = null;
					try {
						do {
							Book book = new Book();
							if (ictag != null) {
								String tagId = ictag.contScanICTag();
								book.setIcTag(tagId);
							}
							dialog = new BookDialog(icTagFrame, book);
							dialog.setModal(true);
							dialog.setLocation(icTagFrame.getLocation());
							dialog.setVisible(true);
							if (!dialog.isCanceled()) {
								tableModel.addBook(book);
								if (ictag != null) {
									ictag.writeICTag(1, 1, book.getShelfId().getBytes());
								}
							}
						} while (!dialog.isCanceled());						
					}
					catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			});
		}
		return jResiterItem;
	}

	/**
	 * This method initializes jDisplayItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJDisplayItem() {
		if (jDisplayItem == null) {
			jDisplayItem = new JMenuItem();
			jDisplayItem.setText(Resource.getString("menu.display"));
			if (ictag == null)
				jDisplayItem.setEnabled(false);
			jDisplayItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					try {
						if (ictag != null) {
							for (;;) {
								String tagId = ictag.contScanICTag();
								String[] tagData = ictag.readICTag(1);
								if (tagData.length > 0) {
									String shelfId = ictag.hexToString(tagData[0], 4, 4);
									JOptionPane.showMessageDialog(icTagFrame, Resource.getString("book.shelfID") + ":" + shelfId);
									break;
								}
							}
						}
					}
					catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			});
		}
		return jDisplayItem;
	}

	/**
	 * This method initializes jQuitItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getJQuitItem() {
		if (jQuitItem == null) {
			jQuitItem = new JMenuItem();
			jQuitItem.setText(Resource.getString("menu.quit"));
			jQuitItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					if (ictag != null) {
						ictag.closeICTag();						
					}
					icTagFrame.dispose();
					System.exit(0);
				}
			});
		}
		return jQuitItem;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getJTable() {
		if (jTable == null) {
			jTable = new JTable(tableModel);
			// select all book
			try {
				List<Book> bookList =DataBaseHelper.selectBook("");
				if (bookList != null)
					tableModel.setBookList(bookList);				
			}
			catch (Exception e) {}

			jTable.addMouseListener(new MouseInputAdapter() {
				public void mouseClicked(MouseEvent e) {
					int row = jTable.rowAtPoint(e.getPoint());
					if (e.getClickCount() == 2) {
						Book book = tableModel.getBook(row);
						BookDialog dialog = new BookDialog(icTagFrame, book);
						dialog.setModal(true);
						dialog.setLocation(icTagFrame.getLocation());
						dialog.setVisible(true);
						if (!dialog.isCanceled() && ictag != null) {
							try {
								ictag.writeICTag(1, 1, book.getShelfId().getBytes());								
							}
							catch (Exception ex) {
								ex.printStackTrace();
							}		
						}
						jTable.repaint();
					}
				}				
			});
		}
		return jTable;
	}


}
