package ictag.util;

import ictag.domain.Book;

import java.io.FileInputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import javax.sql.DataSource;

import org.apache.commons.dbcp.DriverManagerConnectionFactory;
import org.apache.commons.dbcp.PoolableConnectionFactory;
import org.apache.commons.dbcp.PoolingDataSource;
import org.apache.commons.dbutils.handlers.BeanListHandler;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.StackObjectPool;

public class DataBaseHelper {
	/** データソース */
	private static DataSource	dataSource = null;

	private static String	createTableSQL;
	private static String	insertSQL;
	private static String	updateSQL;
	private static String	selectAllSQL;
	private static String	selectWhereSQL;

	private static String	dbDriver;
	private static String	dbUrl;
	private static String	dbUser;
	private static String	dbPasswd;
	
	private static boolean	useIctag = false;
	private static Long		lastId = null;
	private static String	awsAccessKeyID = null;
	
	static {
		ResourceBundle resource = null;
		try {
			resource = new PropertyResourceBundle(new FileInputStream("db.properties"));			
		}
		catch (Exception e) {
			resource = ResourceBundle.getBundle("db");
		}
		dbDriver = resource.getString("db.driver");
		dbUrl = resource.getString("db.url");
		dbUser = resource.getString("db.username");
		dbPasswd = resource.getString("db.password");
		//
		createTableSQL = resource.getString("db.createTable");
		insertSQL = resource.getString("db.insertSQL");
		updateSQL = resource.getString("db.updateSQL");
		selectAllSQL = resource.getString("db.selectAllSQL");
		selectWhereSQL = resource.getString("db.selectWhereSQL");
		useIctag = resource.getString("use.ictag").toLowerCase().equals("true");
		awsAccessKeyID = resource.getString("aws.access.keyID");
	}
	
	/**
	 * 書籍情報をセレクトする。
	 * @param str	検索文字列
	 * @return	書籍情報のリスト
	 * @throws SQLException	SQL例外
	 */
	@SuppressWarnings("unchecked")
	public static List<Book> selectBook(String str) throws SQLException {
		List<Book>	list = null;
		Connection conn = dataSource.getConnection();
		PreparedStatement selectStmt = null;
		if (str.trim().length() == 0) 
			selectStmt = conn.prepareStatement(selectAllSQL);
		else {
			selectStmt = conn.prepareStatement(selectWhereSQL);
			selectStmt.setString(1, str);
			selectStmt.setString(2, str);
		}
		ResultSet	rs = selectStmt.executeQuery();
		BeanListHandler	handler = new BeanListHandler(Book.class);
		list = (List<Book>)handler.handle(rs);
		conn.close();
		return list;
	}
	
	/**
	 * 書籍情報を保存する
	 * @param book	書籍情報
	 * @throws Exception	SQL例外
	 */
	public static void saveBook(Book book) throws Exception {
		Connection conn = dataSource.getConnection();
		PreparedStatement stmt = null;
		if (book.getId() == null) {
			stmt = conn.prepareStatement(insertSQL);
			stmt.setLong(1, getNextID());
		}
		else {
			stmt = conn.prepareStatement(updateSQL);
			stmt.setLong(1, book.getId());
		}		
		stmt.setString(2, book.getIcTag());
		stmt.setString(3, book.getIsbn());
		stmt.setString(4, book.getTitle());
		stmt.setString(5, book.getAuthor());
		stmt.setString(6, book.getPublisher());
		stmt.setString(7, book.getPublicationDate());
		stmt.setString(8, book.getNoPages());
		stmt.setString(9, book.getListPrice());
		stmt.setString(10, book.getShelfId());		
		if (book.getId() != null)
			stmt.setLong(11, book.getId().longValue());	
		stmt.executeUpdate();
		conn.close();						
	}
	
	/**
	 * db.propertiesに記載されているSQL文を使ってテーブルを作成しようと試みる。
	 */
	public static void tryToCreateTable() {
		try {
			dataSource = createDatasource(dbDriver, dbUrl, dbUser, dbPasswd);			
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		try {
			Connection conn = dataSource.getConnection();
			PreparedStatement insStmt = conn.prepareStatement(createTableSQL);
			insStmt.executeUpdate();
			conn.close();				
		}
		catch (Exception e1) {
			// e1.printStackTrace();
		}
	}
	
	
	/**
	 * データソースを作成する。
	 * @param driver	jdbcドライバー
	 * @param url		データベースURL
	 * @param user		ユーザ名
	 * @param password	パスワード
	 * @return	データソース
	 * @throws Exception	例外
	 */
	static DataSource createDatasource(String driver, String url, String user, String password) throws Exception {		
		Class.forName(driver);
		org.apache.commons.dbcp.DriverManagerConnectionFactory conFactory 
			= new DriverManagerConnectionFactory(url, user, password);
		ObjectPool pool = new StackObjectPool();
		new PoolableConnectionFactory(conFactory, pool, null, null, false, true);
		dataSource = new PoolingDataSource(pool);
		return (dataSource);
	}

	/**
	 * ICタグリーダー／ライターを使用するか否かを返す
	 * @return	ICタグリーダー／ライターを使用するか否か
	 */
	public static boolean isUseIctag() {
		return useIctag;
	}

	/**
	 * 次のレコードIDを返す
	 * @return	次のレコードID
	 */
	private static Long getNextID() {
		if (lastId == null) {
			lastId = new Long(0);
			try {
				List<Book>	list = selectBook("");
				for (Book book : list) {
					if (book.getId() > lastId) {
						lastId = book.getId();
					}
				}				
			}
			catch (Exception e) {}
		}
		lastId +=1;
		return (lastId);
	}

	/**
	 * ASW Access Key IDを返す
	 * @return	ASW Access Key ID
	 */
	public static String getAwsAccessKeyID() {
		return awsAccessKeyID;
	}

}
